"use strict";
/*
 * Copyright (c) 2022.
 * Author Peter Placzek (tada5hi)
 * For the full copyright and license information,
 * view the LICENSE file that was distributed with this source code.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupHTTPMiddleware = void 0;
const domains_1 = require("@typescript-auth/domains");
const http_1 = require("@typescript-error/http");
const client_1 = require("@trapi/client");
const utils_1 = require("../../utils");
function setupHTTPMiddleware(context) {
    const tokenCache = (0, utils_1.initTokenCache)(context.redis, context.redisPrefix);
    const tokenAPIClient = new domains_1.TokenAPI(context.http);
    return (req, res, next) => __awaiter(this, void 0, void 0, function* () {
        var _a;
        let { authorization: headerValue } = req.headers;
        if (!headerValue) {
            try {
                let value;
                if (context.cookieHandler) {
                    value = context.cookieHandler(req.cookies);
                }
                else if (((_a = req.cookies) === null || _a === void 0 ? void 0 : _a[domains_1.CookieName.ACCESS_TOKEN]) &&
                    typeof req.cookies[domains_1.CookieName.ACCESS_TOKEN] === 'string') {
                    value = req.cookies[domains_1.CookieName.ACCESS_TOKEN];
                }
                if (value) {
                    headerValue = (0, client_1.stringifyAuthorizationHeader)({ type: 'Bearer', token: value });
                }
            }
            catch (e) {
                // ...
            }
        }
        if (!headerValue) {
            next();
            return;
        }
        const header = (0, client_1.parseAuthorizationHeader)(headerValue);
        if (header.type !== 'Bearer') {
            throw new http_1.BadRequestError('Only Bearer tokens are accepted as authentication method.');
        }
        let data;
        try {
            data = yield (0, utils_1.verifyToken)({
                token: header.token,
                tokenCache,
                tokenAPIClient,
            });
        }
        catch (e) {
            next(e);
            return;
        }
        switch (data.target.kind) {
            case 'robot':
                req.robotId = data.target.entity.id;
                req.robot = data.target.entity;
                break;
            case 'user':
                req.userId = data.target.entity.id;
                req.user = data.target.entity;
                break;
        }
        req.realmId = data.target.entity.realm_id;
        req.token = header.token;
        req.permissions = data.target.permissions;
        req.ability = new domains_1.AbilityManager(data.target.permissions);
        next();
    });
}
exports.setupHTTPMiddleware = setupHTTPMiddleware;
//# sourceMappingURL=module.js.map